# -*- coding: utf-8 -*-

import xml.etree.cElementTree as Et
import requests
from jinja2 import Environment, FileSystemLoader

from .correios_exceptions import CorreiosCEPConnectionErrorException
from .correios_exceptions import CorreiosCEPInvalidCEPException
from .correios_exceptions import CorreiosTimeOutException
from .correios_exceptions import CorreiosCEPTooManyRedirectsException


class Correios:

    URL = 'https://apps.correios.com.br/SigepMasterJPA' \
              '/AtendeClienteService/AtendeCliente?wsdl'

    @staticmethod
    def get_cep(cep: str) -> dict:
        """
        Retorna dos dados do endereço de um dado cep, a saber:
        rua: logradouro do cep
        bairro: bairro do cep
        cidade: cidade do cep
        uf: Abreviacao do estado do cep
        complemento: informações adicionais sobre o cep
        outro: informações variadas sobre o cep como por exemplo o intervalo
        de numero de residência que o mesmo compreende.

        :param cep: string contendo o cep a ser consultado
        :return: dict contendo os dados do endereço do cep consultado.
        """

        xml = Correios._mount_request(Correios._format_cep(cep))

        try:
            response = requests.post(Correios.URL,
                                     data=xml,
                                     headers={
                                         'Content-type': 'text/xml',
                                     },
                                     verify=False)

        except requests.exceptions.Timeout:
            msg = 'Tempo de conexão excedido. Por favor, tente mais tarde.'
            raise CorreiosTimeOutException(msg)

        except requests.exceptions.TooManyRedirects:
            msg = 'Formato de requisição inválido. Por favor, verifique sua ' \
                  'requisição etente novamente'
            raise CorreiosCEPTooManyRedirectsException(msg)

        except requests.ConnectionError:
            msg = 'Erro ao conectar a API. Por favor, verifique sua conexão.'
            raise CorreiosCEPConnectionErrorException(msg)
        else:

            if not response.ok:

                msg = Correios._parse_error(response.text)
                raise CorreiosCEPInvalidCEPException(msg)

            return Correios._parse_response(response.text)

    @staticmethod
    def _format_cep(cep):

        try:
            cep = cep.replace('-', '')
            cep = cep.replace('.', '')

            if not cep.isdigit():
                msg = 'CEP deve ser formado por números!'
                raise CorreiosCEPInvalidCEPException(msg)

            return cep
        except AttributeError:
            msg = 'CEP deve ser do tipo string, ' \
                  'mas o tipo encontrado foi %s!' % type(cep)
            raise CorreiosCEPInvalidCEPException(msg)

    @staticmethod
    def _mount_request(cep):
        env = Environment(loader=FileSystemLoader('pycep_correios/templates'))
        template = env.get_template('consultacep.xml')
        xml = template.render(cep=cep)
        return (xml.replace('\n', '')).replace('\t', '')

    @staticmethod
    def _parse_response(xml):

        end = Et.fromstring(xml).find('.//return')

        address = {
            'rua': end.findtext('end'),
            'bairro': end.findtext('bairro'),
            'cidade': end.findtext('cidade'),
            'uf': end.findtext('uf'),
            'complemento': end.findtext('complemento'),
            'outro': end.findtext('complemento2')
        }

        return address

    @staticmethod
    def _parse_error(xml):
        return Et.fromstring(xml).findtext('.//faultstring')



# -*- coding: utf-8 -*-

__title__ = 'PyCEP Correios'
__version__ = '1.1.5'
__author__ = 'Michell Stuttgart Faria'
__license__ = 'MIT'
__copyright__ = 'Copyright 2015-2017 Michell Stuttgart Faria'

# Version synonym
VERSION = __version__



# -*- coding: utf-8 -*-


class CorreiosCEPException(Exception):
    """ Base class for other exceptions """
    pass


class CorreiosCEPConnectionErrorException(CorreiosCEPException):
    """ Connection error exception """
    pass


class CorreiosCEPTooManyRedirectsException(CorreiosCEPException):
    """ Many redirects exception """
    pass


class CorreiosTimeOutException(CorreiosCEPException):
    """ TimeOut exception """
    pass


class CorreiosCEPInvalidCEPException(CorreiosCEPException):
    """ Invalid CEP Exception. Raised when cep have incorrect
    length or not exist """
    pass



# -*- coding: utf-8 -*-

import requests
from unittest import mock
from unittest import TestCase
from jinja2 import Environment, FileSystemLoader

from pycep_correios.correios import Correios
from pycep_correios.correios_exceptions import \
    CorreiosCEPInvalidCEPException
from pycep_correios.correios_exceptions import \
    CorreiosTimeOutException
from pycep_correios.correios_exceptions import \
    CorreiosCEPTooManyRedirectsException
from pycep_correios.correios_exceptions import \
    CorreiosCEPConnectionErrorException


class TestCorreios(TestCase):

    def setUp(self):

        address = {
            'bairro': 'Asa Norte',
            'cep': '70002900',
            'cidade': 'Brasília',
            'end': 'SBN Quadra 1 Bloco A',
            'id': '0',
            'uf': 'DF',
        }

        self.expected_address = {
            'bairro': address['bairro'],
            'cidade': address['cidade'],
            'complemento': '',
            'outro': '',
            'rua': address['end'],
            'uf': address['uf'],
        }

        self.env = Environment(loader=FileSystemLoader('test/templates'))

        template = self.env.get_template('response.xml')
        xml = template.render(**address)
        self.response_xml = (xml.replace('\n', '')).replace('\t', '')

        template = self.env.get_template('response_error.xml')
        xml = template.render()
        self.response_xml_error = (xml.replace('\n', '')).replace('\t', '')

    @mock.patch('pycep_correios.correios.requests.post')
    def test_get_cep(self, mock_api_call):

        # Aqui realizamos consulta com o CEP correto
        param = {
            'text': self.response_xml,
            'ok': True,
            'status_code': 200,
        }

        mock_api_call.return_value = mock.MagicMock(**param)

        self.assertDictEqual(Correios.get_cep('70002900'),
                             self.expected_address)

        # Aqui realizamos consultas que de alguma forma retornam mensagens de
        # erro
        param = {
            'text': self.response_xml_error,
            'ok': False,
        }

        mock_api_call.return_value = mock.MagicMock(**param)

        self.assertRaises(CorreiosCEPInvalidCEPException,
                          Correios.get_cep, '1232710')

        mock_api_call.side_effect = requests.exceptions.Timeout()
        self.assertRaises(CorreiosTimeOutException,
                          Correios.get_cep, '12345-500')

        mock_api_call.side_effect = requests.exceptions.Timeout()
        self.assertRaises(CorreiosTimeOutException,
                          Correios.get_cep, '12345-500')

        mock_api_call.side_effect = requests.exceptions.TooManyRedirects()
        self.assertRaises(CorreiosCEPTooManyRedirectsException,
                          Correios.get_cep, '12345-500')

        mock_api_call.side_effect = requests.exceptions.ConnectionError()
        self.assertRaises(CorreiosCEPConnectionErrorException,
                          Correios.get_cep, '12345-500')

    def test__format_cep(self):
        self.assertRaises(CorreiosCEPInvalidCEPException,
                          Correios._format_cep, 37503003)

    def test__mount_request(self):
        cep = '37503005'
        template = self.env.get_template('consultacep.xml')
        xml = template.render(cep=cep)
        xml = (xml.replace('\n', '')).replace('\t', '')

        self.assertEqual(xml, Correios._mount_request(cep=cep))

    def test__parse_response(self):
        response = Correios._parse_response(self.response_xml)
        self.assertDictEqual(response, self.expected_address)

    def test__parse_error(self):
        fault = Correios._parse_error(self.response_xml_error)
        self.assertEqual(fault.strip(), 'BUSCA DEFINIDA COMO EXATA, '
                                        '0 CEP DEVE TER 8 DIGITOS')



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name='pycep-correios',
    version='1.1.5',
    keywords='correios development cep',
    packages=find_packages(exclude=['*test*']),
    zip_safe=False,
    url='https://github.com/mstuttgart/pycep-correios',
    license='MIT License',
    author='Michell Stuttgart',
    author_email='michellstut@gmail.com',
    description='Método para busca de dados de CEP no webservice dos Correios',
    long_description=open('README.md', 'r').read(),
    package_data={
        'pycep-correios': ['templates/*xml']
    },
    include_package_data=True,
    install_requires=[
        'requests >= 2.10.0',
        'Jinja2 >= 2.8',
    ],
    test_suite='test',
    tests_require=[
        'coverage',
        'coveralls',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)
